#! /usr/bin/perl
# Validate a submitted copy detection run
# Requires files:
#   ref.durations
#   ref.videos
# Assumes run file name is of form: GROUPID.RUNTYPE.RUNID
# where RUNTYPE must be A, V. or M.

$slash = "/";

$rc = 0;

$runFile = $ARGV[0];

# open run file
open RUN, "$runFile" or
    die "Can't open runfile \"$runFile\": $!\n";

$lineNum = 0;

# Derive run type from file name: GROUPID.RUNTYPE.runId
# where RUNTYPE = a | v | m

$i = rindex($runFile,$slash);
if ($i != -1)
{
    $runFile = substr($runFile,$i);
}
(undef,$runType,undef) = split /\./,$runFile;

# Read valid videoIds into a hash
open VIDEOS, "ref.videos" or die "Can't open ref.video file: $!\n";
while ($line=<VIDEOS>)
{
    ($fileName) = split /\s+/,$line;
    $videos{$fileName} = 1;
}


# Read ref video durations into a hash
open DURATIONS, "ref.durations" or die "Can't open ref.durations file: $!\n";
while ($line=<DURATIONS>)
{
    (undef,$dur,$frames,$fileName,undef) = split /\s+/," $line";
    $durations{$fileName} = $dur;
    #$durations{$fileName} = $frames/25;
}

# Set range of valid query IDs
if ($runType eq "a")
{
    $botValidQueryId = 3001;
    $topValidQueryId = 4407;
    $requiredNumQueries = 1407;
}
elsif ($runType eq "v")
{
    $botValidQueryId = 1;
    $topValidQueryId = 2010;
    $requiredNumQueries = 2010;
} 
elsif ($runType eq "m")
{
    $botValidQueryId =  5001;
    $topValidQueryId = 19070;
    $requiredNumQueries = 14070;
} 

%qIdHash = ();

while ($line =  <RUN>)
{
    $lineNum++;

    chomp $line;

    (undef,$code,$rest) = split /\s+/," $line",3;

    if ($code eq "R")
    {
	($qId,$vId,$firstRefTime,$lastRefTime,$conf,$firstQueryTime) = split /\s+/,$rest;

	# valid queryId for this run type (v, a, m)?
	if ($qId > $topValidQueryId || $qId < $botValidQueryId)
	{
	    print "$lineNum: invalid query id for this run type: $runType\n";
	    $rc = -1;	    
	}

	# valid video file name?
	if (! exists $videos{$vId})
	{
	    print "$lineNum: invalid video id: \"$vId\"\n";
	    $rc = -1;
	}

	# last ref time > first time ref?
	if ($lastRefTime <= $firstRefTime)
	{
	    print "$lineNum: last ref frame precedes first ref frame\n";
	    $rc = -1;
	}

	# first ref time >= 0?
	if ($firstRefTime < 0)
	{
	    print "$lineNum: first ref frame < = 0\n";
	    $rc = -1;
	}

	# first ref time <= end of video?
	if ($firstRefTime > $durations{$vId})
	{
	    print "$lineNum: first ref time ($firstRefTime) beyond end of ref video ($durations{$vId})\n";
	    $rc = -1;
	}

	# last ref time <= end of video?
	if ($lastRefTime > $durations{$vId})
	{
	    print "$lineNum: last ref time ($lasttRefTime) beyond end of ref video ($durations{$vId})\n";
	    $rc = -1;
	}

	# Add result item to hash used to look for possible overlapping items
	if (!exists $overlap{"$qId $vId"})
	{
	    $overlap{"$qId $vId"} = ();
	}

	@list = [$firstRefTime, $lastRefTime];

	push @{ $overlap{"$qId $vId"} }, @list;


	# first query ref <= end of query?

    }
    # Timing info: queryId elapsedQueyProcessingTimeInSeconds
    elsif ($code eq "T")
    {
	($qId,$time) = split /\s+/,$rest;

	# valid queryId for this run type (v, a, a+v)?
	if ($qId < $botValidQueryId || $qId > $topValidQueryId)
	{
	    print "$lineNum: invalid query id for this run type: $runType\n";
	    $rc = -1;
	}
	else
	{
	    $qIdHash{$qId} = 1;
	}
    }
    # runId: string
    elsif ($code eq "I")
    {
	$runId = $rest;
    }
    # operating system: string
    elsif ($code eq "S")
    {
	$opSys = $rest;
    }
    # cpu: string
    elsif ($code eq "C")
    {
	$cpu = $rest;
    }
    # amount of memory: string
    elsif ($code eq "M")
    {
	$memory = $rest;
    }
    # unknown code
    else
    {
	print "$lineNum: Unrecognized code\n";
	$rc = -1;
    }


}

# Check runId present?
if ($runId eq "")
{
    print "run id missing\n";
    $rc = -1;
}

# Check operating system present?
if ($opSys eq "")
{
    print "operating system info missing\n";
    $rc = -1;
}

# Check cpu present?
if ($cpu eq "")
{
    print "cpu type missing\n";
    $rc = -1;
}

# Check memory present?
if ($memory eq "")
{
    print "memory size missing\n";
    $rc = -1;
}

# Check timing for all queries present?
$actualNumQueries = scalar keys %qIdHash;
if ($actualNumQueries != $requiredNumQueries)
{
    print "Missing/extra Timing info for queries: timing for $requiredNumQueries queries required, $actualNumQueries submitted\n";
    $rc = -1;
}

# Check no overlapping result items
foreach $queryVideo (sort keys %overlap)
{
    $maxI = scalar @{$overlap{$queryVideo}};
    
    for ($i=0;$i<$maxI;$i++)
    {
	for ($j=$i+1;$j<$maxI;$j++)
	{

	    if (
	        ((@{@{$overlap{$queryVideo}}[$i]}[0] >=  @{@{$overlap{$queryVideo}}[$j]}[0]) &&
	         (@{@{$overlap{$queryVideo}}[$i]}[0] <=  @{@{$overlap{$queryVideo}}[$j]}[1]))
	    ||
	        ((@{@{$overlap{$queryVideo}}[$i]}[1] >=  @{@{$overlap{$queryVideo}}[$j]}[0]) &&
	         (@{@{$overlap{$queryVideo}}[$i]}[1] <=  @{@{$overlap{$queryVideo}}[$j]}[1]))
	       )

	    {
		print "Overlapping results items for Query $queryVideo: (@{@{$overlap{$queryVideo}}[$i]}) (@{@{$overlap{$queryVideo}}[$j]}) \n";
		$rc = -1;
	    }
	}
    }

    #print "\n";
}

exit $rc;
